
# q-expansion of newform 8030.2.a.q, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8030
weight = 2
poly_data = [-4, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1, 0], [0, 1], [-1, 0], [-2, -1], [1, 0], [-2, 0], [-1, 2], [1, 0], [1, -3], [-7, 0], [3, -2], [1, -2], [3, -3], [3, -1], [8, -2], [-4, 5], [-6, 2], [-8, 1], [-12, 0], [4, -7], [-1, 0], [-2, 0], [4, 0], [1, 0], [-8, -2], [1, 1], [8, -6], [-7, 3], [2, 0], [-10, 0], [-3, 7], [-16, 3], [-18, 0], [-4, 0], [6, 3], [0, 0], [-10, 3], [16, -1], [-8, 9], [7, -5], [-2, 0], [-4, 4], [0, -8], [13, 0], [20, -2], [-1, 0], [-13, 2], [-13, 3], [12, 0], [5, 3], [2, -9], [3, -1], [0, 6], [-21, 1], [4, 2], [0, -5], [-24, 0], [8, -4], [-10, -2], [-2, 12], [-6, -2], [15, -7], [-1, 1], [-14, 5], [-7, 13], [17, 4], [-10, 8], [13, 6], [26, 2], [2, 8], [2, 6], [10, 0], [-8, -6], [2, -10], [-8, 10], [-32, 0], [-30, -2], [-6, 12], [-1, 0], [16, -2], [-7, 9], [-11, -1], [-1, 1], [6, 2], [0, -12], [1, -9], [20, -10], [-10, 3], [-38, -1], [12, 8], [-1, 12], [-32, 6], [-3, 11], [10, -7], [-31, 3], [19, 1], [-24, 6], [-2, 4], [10, 4], [-7, -4], [18, -4], [-19, 7], [-16, -6], [24, -4], [10, -21], [3, 15], [-6, 13], [-22, 4], [-13, -6], [4, 6], [12, -7], [12, 8], [21, -11], [17, 5], [19, -16], [17, 10], [-1, 4], [4, 2], [30, -5], [15, 0], [10, 4], [-36, -5], [32, 2], [5, -20], [26, -16], [-1, -16], [-18, 0], [-15, 0], [-21, 7], [-29, 11], [32, 4], [-24, 13], [-4, 1], [22, 4], [22, 10], [0, 0], [8, -7], [-38, -6], [35, 3], [29, -13], [16, 11], [12, -2], [40, -4], [0, -8], [-6, 12], [10, -6], [-17, 15], [-28, -9], [-4, 4], [-32, 2], [6, 0], [4, 14], [-8, 13], [20, 4], [-28, 11], [-6, 15], [-29, 3], [-22, -1], [-40, -6], [-12, 19], [-12, -13], [16, -7], [29, 12], [4, -2], [-1, -1], [4, -20], [16, 0], [7, -11], [-2, 18], [34, 5], [16, 1], [0, -16], [-14, 14], [23, -22], [-23, -5], [-11, -2], [18, -6], [-17, -3], [-15, -8], [13, -6], [24, -26], [0, 1], [-11, 0], [-40, 7], [5, 7], [-31, 5], [14, -4], [-1, 9], [24, -16], [29, -17], [-7, -9], [-1, -13], [-12, -8], [-21, 1], [-14, 6], [-6, 2], [-60, 4], [30, -20], [-6, -8], [-17, -2], [-4, -13], [17, 13], [15, -6], [-61, -1], [-8, 4], [39, 0], [-20, 12], [20, -10], [-15, 11], [13, 9], [-30, 17], [-23, -7], [28, -16], [17, -24], [-8, 16], [6, -5], [56, -5], [36, -10], [26, -4], [-10, 5], [-28, -10], [14, 8], [0, 11], [-48, 14], [9, -15], [-26, 17], [-16, -20], [-36, -5], [40, -20], [5, -13], [16, 10], [-8, 7], [15, -1], [14, 18], [12, 8], [-7, 24], [24, -15], [-16, -6], [42, 0], [9, -12], [-27, -12], [-15, -1], [-32, -10], [-10, -4], [-21, 11], [36, -2], [8, -12], [-20, 12], [-19, 16], [31, -2], [-5, -1], [28, 16], [-71, 1], [-11, -22], [34, 0], [-4, 27], [-57, -5], [-13, -19], [34, 4], [-6, 25], [-33, -11], [28, -26], [-19, -18], [-48, 12], [40, 2], [-38, -14], [2, 16], [-24, -9], [-23, 25], [39, -9], [-35, 7], [25, -32], [12, -16], [-27, 5], [4, -9], [-52, 18], [-8, 12], [-44, 10], [39, -21], [-44, 17], [24, 3], [-26, -10], [28, -12], [2, 27], [0, -32], [-2, -7], [-4, -16], [56, 0], [46, -1], [49, 1], [34, 10], [12, -8], [35, -14], [22, -37], [22, 2], [-19, 1], [-8, -8], [37, -24], [21, 11], [-20, 22], [25, -22], [-36, 18], [63, -11], [0, 12], [4, 6], [16, 0], [-1, -7], [-2, 12], [-23, -4], [-47, 15], [21, 23], [-70, 10], [-1, -21], [9, 2], [14, 11], [-28, -8], [36, -4], [8, -22], [-30, -6], [-33, 3], [-36, -19], [-37, 19], [14, -24], [24, -4], [24, 21], [60, -7], [6, 18], [-17, 9], [28, 13], [-7, -21], [31, -18], [32, 14], [-10, -4], [-21, 25], [28, -3], [34, -9], [60, 6], [-9, -22], [-21, 6], [-56, -5], [-38, 28], [49, -17], [-62, -11], [18, -1], [27, 24], [-51, 10], [-46, 26], [8, 13], [-40, -18], [-50, 5], [-63, 12], [-32, -14], [-26, -20], [-24, -5], [2, -7], [34, 7], [-19, -8], [16, -25], [42, 16], [66, 0], [-56, 14], [-25, -1], [-29, 27], [27, -36], [57, -18], [52, -14], [40, -12], [34, -5], [-12, 24], [-43, -21], [35, -15], [-56, 24], [-4, 1], [-60, 10], [-51, 29], [-4, -2], [-46, 6], [-20, -5], [-7, 19], [-18, 1], [-3, 3], [-70, 16], [-39, 3], [41, -26], [-22, -6], [23, -9], [-23, 39], [26, 7], [-28, 8], [52, -16], [-39, 4], [9, 28], [-57, 19], [-26, -2], [7, -7], [-2, -20], [-24, -12], [-34, 31], [-66, 16], [46, 9], [58, -7], [9, -15], [4, 28], [-54, 2], [50, 4], [-60, -10], [13, -28], [-1, 33], [-59, -12], [-14, 26], [-55, 1], [26, 30], [-28, 36], [-85, -3], [9, 9], [-17, -13], [24, 8], [-92, 2], [-50, 6], [-36, 5], [65, 5], [-63, 15], [6, -24], [-44, -19], [-31, -1], [-22, 15], [4, -3], [34, -18], [-24, -8], [38, 12]]
